// src/entry.rs
#[no_mangle]
#[link_section = ".text.init"]
unsafe extern "C" fn _entry() {
    use core::arch::asm;
    // let id = riscv::register::mhartid::read();
    // write_tp(&id);
    // TODO set up stack for all harts
    // TODO set up CSRs for all harts
    // if id != 0 { abort(); }

    asm!(
        /* Global pointer register `gp`
        
        Push current option stack to temporarily disable relaxation, load
        _global_pointer symbol (provided by linker), then pop option stack.

        Important to keep relaxation off for this instruction, so that
        the instruction is emitted as:
            1:
                auipc gp, %pcrel_hi(_global_pointer)
                addi gp, gp, %pcrel_lo(1b)"
        instead of:
                mv gp, gp
        which would do nothing. */
        ".option push",
        ".option norelax",
        "la gp, _global_pointer",
        ".option pop",
    );

    riscv::register::satp::write(0);

    let id = riscv::register::mhartid::read();
    if id != 0 { crate::abort(); }

    // Clear BSS section
    asm!(
        "la t0, _bss_start",
        "la t1, _bss_end",
        "bgeu t0, t1, 2f",
    "1:",
        "sb zero, 0(t0)",
        "addi t0, t0, 1",
        "bne t0, t1, 1b",

    "2:",
        "la sp, _stack_end",

        "li t0, (0b11 << 11) | (1 << 13)",
        "csrw mstatus, t0",
        "csrw mie, x0",
        
    );

    // TODO do hardware inits and wake other harts
}

#[inline]
unsafe fn write_tp(id: &usize) {
    use core::arch::asm;
    asm!(
        "csrw mhartid, {id}",
        id = in(reg) id,
    );
}